/* startnet_pearl.rexx                                                    */
/*                                                                        */
/* Version 1.0, 11 December 1993, by P.J. Rovero                          */
/*         1.2, 19 December 1993                                          */
/*         1.3, 30 December 1993, by K. Raquel Sanborn (raquels@uhunix)   */
/*         1.4, 3  January  1994, Fix reboot Amiga without lose of CSLIP  */
/*         1.5, 5  January  1994, Fix mounting of APIPE:                  */
/*         1.6, 6  January  1994, Add BREAK C support and "PROTECT +S"    */
/*         1.7, 6  January  1994, Add FAILTAT 21 for mount >NIL: APIPE:   */
/*                                                                        */
/* An ARexx script that will dial a terminal server, connect, determine   */
/* the dynamically assigned IP address, and start the proper components   */
/* of AmiTCP 2.x.  The script uses rexxserdev.library v2.0 by Joseph M.   */
/* Stivaletta.                                                            */
/*                                                                        */
/* Inspired by Dave Bolen's REXX/2 SLIPUP.CMD for IBM OS/2 TCPIP 1.2.1    */
/*                                                                        */
/* ---------------------------------------------------------------------- */

/* Start the required libraries                      */

    say 'Opening libraries...'

    check = addlib('rexxsupport.library', 0, -30, 0)
    if (check == 0) then say 'rexxsupport library did not open...'

    check = addlib('rexxarplib.library',  0, -30, 0)
    if (check == 0) then say 'rexxarplib did not open...'

    check = addlib('rexxserdev.library',  0, -30, 0)
    if (check == 0) then say 'rexxserdev did not open...'


/* Define modem strings and other initialization stuff     */

   say 'Initializing...'

    cr              = '0d'x
    crlf            = '0d0a'x
    lf              = '0a'x
    at_cmd          = 'AT'||cr
    response        = 'OK'
    dialstr         = 'ATDTxxx-xxxx'||cr /* Put your telephone number here */
    pearl_port      = 'Pearl port'
    slip_cmd        = 'slip'||cr
    username        = 'your_name'||cr       /* Put your user name here */
    passwd          = 'your_passwd'||cr     /* Put your passwd here */
    slip_ip         = 'My IP'
    host_ip         = 'Host IP'

/* Open the serial device and talk to the modem                          */
/* Mine happens to be a 57600 baud Hayes-compatible on serial unit 0     */

    say 'Setting up serial device...'
    serhandle = SerOpen('serial.device', 0)
    check     = SerClear(serhandle)
    check     = SerSetParms( serhandle, 57600, 8, N, 1, 0 )


/* Clean up on "BREAK C" */

    buffer = allocmem( 4096 )
    junk   = c2d( buffer )
    signal on break_c


/* Dial & wait for a connection and the modem status string */

    say 'Dialing the terminal server... '
    check    = 0
    do forever
        if (check == 0) then
	do
	    result = SerWrite( serhandle, dialstr, length( dialstr))
            check = 1
            totalstr = ''
        end
	status   = SerQuery( serhandle )
        parse upper var status valid bytes_read status_bits .
        rcvdstr  = SerRead( serhandle, junk, bytes_read )
        totalstr = totalstr||rcvdstr
        check    = modem_query(totalstr)
        if (check == 100) then leave
        result   = delay(100)
    end

    totalstr = ''

   say 'Modem has connected to the terminal server...'
   check = delay(50)
   check = SerFlush( serhandle, 'R')
   check = delay(50)


/* send CR to terminal server to make sure we get pearl port number   */

   say 'Sent CR to server...'
   check = SerWrite( serhandle, cr, length( cr ))
   call wait_str('pearl*')
   parse var rcvdstr 'pearl*' pearl_port ': '
   do until (length(pearl_port) = 3)
       pearl_port = '0'||pearl_port
   end
   say 'Pearl port is '||pearl_port


/* send SLIP<cr> to terminal server */

   say 'Terminal server prompt received, now sending slip command...'
   check = SerWrite( serhandle, slip_cmd, length( slip_cmd))
   check = SerRead(  serhandle, junk, length(slip_cmd))


/* Wait for 'Username: ' prompt */

   call wait_str('Username: ')
   say 'Received user prompt...'
   check = SerWrite( serhandle, username, length( username ))
   check = SerRead(  serhandle, junk, length(username))
   say 'Sent Username, now waiting for password prompt...'


/* Wait for 'Password:' prompt */

    call wait_str('Password: ')
    say 'Received password prompt...'
    check = SerWrite( serhandle, passwd, length( passwd ))
    say 'Sent password, now waiting for IP addresses...'
    address command
    'wait 2 secs'
    call wait_str('Annex')
    say 'SLIP address received from terminal server...'


/* Ok, now we have all the data we need to start slip            */
/* Close down the serial port but leave the line up (ignore DTR) */

    check = SerClose( serhandle )
    result = startslip()
    check = freemem(buffer,4096)
exit result
/* ------------------------------------------------------------- */
/* Exit point for "Break C" or ^C                                */
/* ------------------------------------------------------------- */

break_c:
    say 'Trying to clean up after ^C...'
    check = SerClose( serhandle )
    check = freemem(buffer,4096)
exit 20
/* ------------------------------------------------------------- */
/* procedure startslip                                           */
/* ------------------------------------------------------------- */
/* Start and configure AmiTCP for SLIP                           */

startslip:

    parse var rcvdstr . 'Annex address is ' a '.' b '.' c '.' d '.'.
    host_ip = a||'.'||b||'.'||c||'.'||d
    say 'The Annex address is :'||host_ip||':'

    parse var rcvdstr . 'Your address is ' a '.' b '.' c '.' d '.'.
    slip_ip = a||'.'||b||'.'||c||'.'||d
    say 'Your SLIP address is :'||slip_ip||':'


/* Start AmiTCP                                                  */

    say 'Starting AmiTCP...'
    address command
    'setenv HOSTNAME nts'||pearl_port||'.dialup.hawaii.edu'
    'copy ENV:HOSTNAME ENVARC:'


/* Set myhost and env:sana2/slip0.config up to reflect ip_addr   */
/*   *** Be sure to set this for *your* machine and domain ***   */

    check    = open(temp,'ENV:Sana2/slip0.config','W')
    temp_str = 'serial.device 0 57600 '||slip_ip||' CD 7WIRE COMP MTU=256'
    check    = writeln(temp,temp_str)
    check    = close(temp)
    'copy ENV:Sana2/slip0.config ENVARC:Sana2'


/*   *** Be sure to set this for *your* machine and domain ***   */

    'copy AmiTCP:db/pearl/netdb-myhost AmiTCP:db'
    check    = open(temp,'AmiTCP:db/netdb-myhost','A')
    temp_str = 'HOST '||slip_ip||' nts'||pearl_port||'.dialup.hawaii.edu arwen'||lf
    check    = writeln(temp,temp_str)
    check    = close(temp)


/* Write the "Startnet_pearl.cont" file */

    check    = open(temp,'AmiTCP:bin/startnet_pearl.cont','W')
    check    = writeln(temp,'failat 21')
    check    = writeln(temp,'mount >NIL: APIPE:')
    check    = writeln(temp,'failat 20')
    check    = writeln(temp,'run AmiTCP:AmiTCP')
    check    = writeln(temp,'sys:rexxc/WaitForPort AMITCP')
    check    = writeln(temp,'wait 2 secs')
    check    = writeln(temp,'AmiTCP:bin/ifconfig lo/0 127.1')
    temp_str = 'AmiTCP:bin/ifconfig devs:networks/slip.device/0 '||slip_ip||' '||host_ip||' netmask 255.255.255.0 broadcast 128.171.33.255'
    check    = writeln(temp,temp_str)
    temp_str = 'AmiTCP:bin/route add nts'||pearl_port||'.dialup.hawaii.edu localhost'
    check    = writeln(temp,temp_str)
    check    = writeln(temp,'AmiTCP:bin/route add default 128.171.33.1')
    check    = writeln(temp,'run AmiTCP:bin/inetd')
    check    = close(temp)
    'protect AmiTCP:bin/startnet_pearl.cont +s'

/*    'mount TCP: from devs:Inet-Mountlist' */

    say 'AmiTCP:bin/startnet_pearl.cont file written...'
    'AmiTCP:bin/startnet_pearl.cont'

return 0

/* ------------------------------------------------------------- */


/* ------------------------------------------------------------- */
/* Subroutine wait_str  returns when argument string is received */
/* ------------------------------------------------------------- */
wait_str:

    teststr = arg(1)
    check = 0
    do until (check ~= 0)
        status = SerQuery( serhandle )
        parse upper var status valid bytes_read status_bits .
        rcvdstr = SerRead( serhandle, junk, bytes_read )
        check   = pos(teststr,rcvdstr)
        test    = delay(50)
    end
return
/* ------------------------------------------------------------- */

/* ------------------------------------------------------------- */
/* Subroutine modem_query   determine modem status               */
/* ------------------------------------------------------------- */
modem_query: procedure

    teststr = arg(1)
    connectstr       = 'CONNECT'
    busystr          = 'BUSY'
    nocarrierstr     = 'NO CARRIER'

    test = pos(connectstr, teststr)
    if (test ~= 0) then return 100

    test = pos(busystr, teststr)
    if (test ~= 0) then return 0

    test = pos(nocarrierstr, teststr)
    if (test ~= 0) then return 0

return 200
/* ------------------------------------------------------------- */

